@@ -556,6 +556,7 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
 {
   ClutterFlowLayoutPrivate *priv = CLUTTER_FLOW_LAYOUT (manager)->priv;
   GList *l, *children = clutter_container_get_children (container);
+  gfloat x_off, y_off;
   gfloat avail_width, avail_height;
   gfloat item_x, item_y;
   gint line_item_count;
@@ -565,6 +566,7 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
   if (children == NULL)
     return;
 
+  clutter_actor_box_get_origin (allocation, &x_off, &y_off);
   clutter_actor_box_get_size (allocation, &avail_width, &avail_height);
 
   /* blow the cached preferred size and re-compute with the given
@@ -585,7 +587,8 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
   items_per_line = compute_lines (CLUTTER_FLOW_LAYOUT (manager),
                                   avail_width, avail_height);
 
-  item_x = item_y = 0;
+  item_x = x_off;
+  item_y = y_off;
 
   line_item_count = 0;
   line_index = 0;
@@ -616,7 +619,7 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
               line_item_count = 0;
               line_index += 1;
 
-              item_x = 0;
+              item_x = x_off;
             }
 
           new_x = ((line_item_count + 1) * (avail_width + priv->col_spacing))
@@ -655,7 +658,7 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
               line_item_count = 0;
               line_index += 1;
 
-              item_y = 0;
+              item_y = y_off;
             }
 
           new_y = ((line_item_count + 1) * (avail_height + priv->row_spacing))
@@ -999,9 +1002,9 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
   gobject_class->finalize = clutter_flow_layout_finalize;
   gobject_class->set_property = clutter_flow_layout_set_property;
   gobject_class->get_property = clutter_flow_layout_get_property;
-  _clutter_object_class_install_properties (gobject_class,
-                                            N_PROPERTIES,
-                                            flow_properties);
+  g_object_class_install_properties (gobject_class,
+                                     N_PROPERTIES,
+                                     flow_properties);
 }
 
 static void
@@ -1087,7 +1090,7 @@ clutter_flow_layout_set_orientation (ClutterFlowLayout      *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      _clutter_notify_by_pspec (G_OBJECT (layout),
+      g_object_notify_by_pspec (G_OBJECT (layout),
                                 flow_properties[PROP_ORIENTATION]);
     }
 }
@@ -1140,7 +1143,7 @@ clutter_flow_layout_set_homogeneous (ClutterFlowLayout *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      _clutter_notify_by_pspec (G_OBJECT (layout),
+      g_object_notify_by_pspec (G_OBJECT (layout),
                                 flow_properties[PROP_HOMOGENEOUS]);
     }
 }
@@ -1191,7 +1194,7 @@ clutter_flow_layout_set_column_spacing (ClutterFlowLayout *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      _clutter_notify_by_pspec (G_OBJECT (layout),
+      g_object_notify_by_pspec (G_OBJECT (layout),
                                 flow_properties[PROP_COLUMN_SPACING]);
     }
 }
@@ -1243,7 +1246,7 @@ clutter_flow_layout_set_row_spacing (ClutterFlowLayout *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      _clutter_notify_by_pspec (G_OBJECT (layout),
+      g_object_notify_by_pspec (G_OBJECT (layout),
                                 flow_properties[PROP_ROW_SPACING]);
     }
 }
@@ -1313,11 +1316,11 @@ clutter_flow_layout_set_column_width (ClutterFlowLayout *layout,
     }
 
   if (notify_min)
-    _clutter_notify_by_pspec (G_OBJECT (layout),
+    g_object_notify_by_pspec (G_OBJECT (layout),
                               flow_properties[PROP_MIN_COLUMN_WIDTH]);
 
   if (notify_max)
-    _clutter_notify_by_pspec (G_OBJECT (layout),
+    g_object_notify_by_pspec (G_OBJECT (layout),
                               flow_properties[PROP_MAX_COLUMN_WIDTH]);
 
   g_object_thaw_notify (G_OBJECT (layout));
@@ -1393,11 +1396,11 @@ clutter_flow_layout_set_row_height (ClutterFlowLayout *layout,
     }
 
   if (notify_min)
-    _clutter_notify_by_pspec (G_OBJECT (layout),
+    g_object_notify_by_pspec (G_OBJECT (layout),
                               flow_properties[PROP_MIN_ROW_HEGHT]);
 
   if (notify_max)
-    _clutter_notify_by_pspec (G_OBJECT (layout),
+    g_object_notify_by_pspec (G_OBJECT (layout),
                               flow_properties[PROP_MAX_ROW_HEIGHT]);
 
   g_object_thaw_notify (G_OBJECT (layout));
