@@ -31,6 +31,7 @@
 #include <string.h>
 
 #include <glib-object.h>
+#include <math.h>
 
 #include "clutter-actor-private.h"
 #include "clutter-paint-volume-private.h"
@@ -289,16 +290,15 @@ clutter_paint_volume_set_width (ClutterPaintVolume *pv,
  * around the volume. It returns the size of that bounding box as
  * measured along the x-axis.
  *
- * <note>If, for example, clutter_actor_get_transformed_paint_volume()
+ * <note><para>If, for example, clutter_actor_get_transformed_paint_volume()
  * is used to transform a 2D child actor that is 100px wide, 100px
  * high and 0px deep into container coordinates then the width might
  * not simply be 100px if the child actor has a 3D rotation applied to
- * it.
- *
- * Remember; after clutter_actor_get_transformed_paint_volume() is
+ * it.</para>
+ * <para>Remember; after clutter_actor_get_transformed_paint_volume() is
  * used then a transformed child volume will be defined relative to the
  * ancestor container actor and so a 2D child actor
- * can have a 3D bounding volume.</note>
+ * can have a 3D bounding volume.</para></note>
  *
  * <note>There are no accuracy guarantees for the reported width,
  * except that it must always be >= to the true width. This is
@@ -381,16 +381,15 @@ clutter_paint_volume_set_height (ClutterPaintVolume *pv,
  * around the volume. It returns the size of that bounding box as
  * measured along the y-axis.
  *
- * <note>If, for example, clutter_actor_get_transformed_paint_volume()
+ * <note><para>If, for example, clutter_actor_get_transformed_paint_volume()
  * is used to transform a 2D child actor that is 100px wide, 100px
  * high and 0px deep into container coordinates then the height might
  * not simply be 100px if the child actor has a 3D rotation applied to
- * it.
- *
- * Remember; after clutter_actor_get_transformed_paint_volume() is
+ * it.</para>
+ * <para>Remember; after clutter_actor_get_transformed_paint_volume() is
  * used then a transformed child volume will be defined relative to the
  * ancestor container actor and so a 2D child actor
- * can have a 3D bounding volume.</note>
+ * can have a 3D bounding volume.</para></note>
  *
  * <note>There are no accuracy guarantees for the reported height,
  * except that it must always be >= to the true height. This is
@@ -474,16 +473,15 @@ clutter_paint_volume_set_depth (ClutterPaintVolume *pv,
  * around the volume. It returns the size of that bounding box as
  * measured along the z-axis.
  *
- * <note>If, for example, clutter_actor_get_transformed_paint_volume()
+ * <note><para>If, for example, clutter_actor_get_transformed_paint_volume()
  * is used to transform a 2D child actor that is 100px wide, 100px
  * high and 0px deep into container coordinates then the depth might
  * not simply be 0px if the child actor has a 3D rotation applied to
- * it.
- *
- * Remember; after clutter_actor_get_transformed_paint_volume() is
+ * it.</para>
+ * <para>Remember; after clutter_actor_get_transformed_paint_volume() is
  * used then the transformed volume will be defined relative to the
  * container actor and in container coordinates a 2D child actor
- * can have a 3D bounding volume.</note>
+ * can have a 3D bounding volume.</para></note>
  *
  * <note>There are no accuracy guarantees for the reported depth,
  * except that it must always be >= to the true depth. This is
@@ -952,7 +950,7 @@ _clutter_actor_set_default_paint_volume (ClutterActor       *self,
                                          GType               check_gtype,
                                          ClutterPaintVolume *volume)
 {
-  ClutterGeometry geometry = { 0, };
+  ClutterActorBox box;
 
   if (check_gtype != G_TYPE_INVALID)
     {
@@ -967,14 +965,14 @@ _clutter_actor_set_default_paint_volume (ClutterActor       *self,
   if (!clutter_actor_has_allocation (self))
     return FALSE;
 
-  clutter_actor_get_allocation_geometry (self, &geometry);
+  clutter_actor_get_allocation_box (self, &box);
 
   /* a zero-sized actor has no paint volume */
-  if (geometry.width == 0 || geometry.height == 0)
+  if (box.x1 == box.x2 || box.y1 == box.y2)
     return FALSE;
 
-  clutter_paint_volume_set_width (volume, geometry.width);
-  clutter_paint_volume_set_height (volume, geometry.height);
+  clutter_paint_volume_set_width (volume, box.x2 - box.x1);
+  clutter_paint_volume_set_height (volume, box.y2 - box.y1);
 
   return TRUE;
 }
@@ -1098,6 +1096,8 @@ _clutter_paint_volume_get_stage_paint_box (ClutterPaintVolume *pv,
   CoglMatrix modelview;
   CoglMatrix projection;
   float viewport[4];
+  float width;
+  float height;
 
   _clutter_paint_volume_copy_static (pv, &projected_pv);
 
@@ -1121,7 +1121,51 @@ _clutter_paint_volume_get_stage_paint_box (ClutterPaintVolume *pv,
                                  viewport);
 
   _clutter_paint_volume_get_bounding_box (&projected_pv, box);
-  clutter_actor_box_clamp_to_pixel (box);
+
+  /* The aim here is that for a given rectangle defined with floating point
+   * coordinates we want to determine a stable quantized size in pixels
+   * that doesn't vary due to the original box's sub-pixel position.
+   *
+   * The reason this is important is because effects will use this
+   * API to determine the size of offscreen framebuffers and so for
+   * a fixed-size object that may be animated accross the screen we
+   * want to make sure that the stage paint-box has an equally stable
+   * size so that effects aren't made to continuously re-allocate
+   * a corresponding fbo.
+   *
+   * The other thing we consider is that the calculation of this box is
+   * subject to floating point precision issues that might be slightly
+   * different to the precision issues involved with actually painting the
+   * actor, which might result in painting slightly leaking outside the
+   * user's calculated paint-volume. For this we simply aim to pad out the
+   * paint-volume by at least half a pixel all the way around.
+   */
+  width = box->x2 - box->x1;
+  height = box->y2 - box->y1;
+  width = CLUTTER_NEARBYINT (width);
+  height = CLUTTER_NEARBYINT (height);
+  /* XXX: NB the width/height may now be up to 0.5px too small so we
+   * must also pad by 0.25px all around to account for this. In total we
+   * must padd by at least 0.75px around all sides. */
+
+  /* XXX: The furthest that we can overshoot the bottom right corner by
+   * here is 1.75px in total if you consider that the 0.75 padding could
+   * just cross an integer boundary and so ceil will effectively add 1.
+   */
+  box->x2 = ceilf (box->x2 + 0.75);
+  box->y2 = ceilf (box->y2 + 0.75);
+
+  /* Now we redefine the top-left relative to the bottom right based on the
+   * rounded width/height determined above + a constant so that the overall
+   * size of the box will be stable and not dependant on the box's
+   * position.
+   *
+   * Adding 3px to the width/height will ensure we cover the maximum of
+   * 1.75px padding on the bottom/right and still ensure we have > 0.75px
+   * padding on the top/left.
+   */
+  box->x1 = box->x2 - width - 3;
+  box->y1 = box->y2 - height - 3;
 
   clutter_paint_volume_free (&projected_pv);
 }
